
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.SIMON96_pkg.all;

entity SIMON96 is
    generic (
        G_RNDS      : integer := SIMON_ROUNDS);
    port(
        clk         : in  std_logic;
        rst         : in  std_logic;
        din         : in  std_logic_vector(SIMON_BLOCK_SIZE-1 downto 0);
        key         : in  std_logic_vector(SIMON_KEY_SIZE  -1 downto 0);
        dout        : out std_logic_vector(SIMON_BLOCK_SIZE-1 downto 0);

        start       : in  std_logic;
        ready       : out std_logic;
        done        : out std_logic
    );
end SIMON96;

-------------------------------------------------------------------------------
--! @brief  Architecture definition of SIMON_Enc_KOF
-------------------------------------------------------------------------------

architecture structure of SIMON96 is
    signal sel_in       : std_logic;
    signal en_in        : std_logic;
    signal round        : std_logic_vector(ROUNDS_CNT_LEN-1 downto 0);

begin

    u_dp: entity work.SIMON96_Datapath(structure)
    port map (  clk         => clk,
                rst         => rst,
                --! Data
                din         => din,
                key         => key,
                dout        => dout,
                --! Control
                round       => round,
                sel_in      => sel_in,
                en_in       => en_in);

    u_ctrl: entity work.SIMON96_Control(behav)
    generic map (G_RNDS     => G_RNDS)
    port map (  clk         => clk,
                rst         => rst,
                --! External
                start       => start,
                ready       => ready,
                done        => done,
                --! Internal
                round       => round,
                sel_in      => sel_in,
                en_in       => en_in);

end structure;