
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.SIMON96_pkg.all;

entity SIMON96_Datapath is
    generic( 
        G_BLK_SIZE  : integer := SIMON_BLOCK_SIZE; 
        G_KEY_SIZE  : integer := SIMON_KEY_SIZE );
    port(
        clk         : in  std_logic;
        rst         : in  std_logic;
        din         : in  std_logic_vector(G_BLK_SIZE  -1 downto 0);
        key         : in  std_logic_vector(G_KEY_SIZE  -1 downto 0);
        dout        : out std_logic_vector(G_BLK_SIZE  -1 downto 0);
        
        round       : in  std_logic_vector(ROUNDS_CNT_LEN-1 downto 0);
        sel_in      : in  std_logic;
        en_in       : in  std_logic);
end SIMON96_Datapath;

-------------------------------------------------------------------------------
--! @brief  Architecture definition of SIMON_Datapath
-------------------------------------------------------------------------------

architecture structure of SIMON96_Datapath is
    signal from_reg         : std_logic_vector(G_BLK_SIZE  -1 downto 0);    
    signal from_round_fdb   : std_logic_vector(G_BLK_SIZE  -1 downto 0);
    signal ki               : std_logic_vector(G_KEY_SIZE  -1 downto 0);
    signal ko               : std_logic_vector(G_KEY_SIZE  -1 downto 0);
    signal rkey             : std_logic_vector(G_KEY_SIZE  -1 downto 0);
begin

    p_reg: process(clk)
    begin
        if rising_edge(clk) then
            if en_in = '1' then
                if sel_in = '1' then
                    from_reg <= din;                     
                else
                    from_reg <= from_round_fdb;
                end if;
                rkey    <= ko;
            end if;
        end if;
    end process;
    
    u_round: entity work.SIMON96_Round(behavior)
    port map (
        din         => from_reg,
        rkey        => rkey,
        dout_fdb    => from_round_fdb
        );
    dout <= from_round_fdb;
    --! Key Expansion
    ki <= key when sel_in = '1' else rkey;    
    u_keyexp: entity work.SIMON96_KeyUpdate(behavior)
    port map (
        round       => round,
        ki          => ki,
        ko          => ko);
        
end structure;