
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.SIMON96_pkg.all;

entity SIMON96_KeyUpdate is
generic( 
        BS  : integer := SIMON_BLOCK_SIZE; 
        KS  : integer := SIMON_KEY_SIZE; 
        WS  : integer := SIMON_WORD_SIZE );
    port (
        round   : in  std_logic_vector(ROUNDS_CNT_LEN-1 downto 0);
        ki      : in  std_logic_vector(KS  -1 downto 0);
        ko      : out std_logic_vector(KS  -1 downto 0)
    );
end SIMON96_KeyUpdate;

-------------------------------------------------------------------------------
--! @brief  Architecture definition of SIMON_KeyUpdate
-------------------------------------------------------------------------------

architecture behavior of SIMON96_KeyUpdate is
    constant z       : std_logic_vector(61 downto 0) := "10101111011100000011010010011000101000010001111110010110110011"; 

begin
     -- m = 2
     -- k(i-1) = ki(KS-1 downto KS-WS) 
     -- k(i-m) = ki(WS-1 downto 0) 
     -- krr3 <= ki(KS-WS+2 downto KS-WS) & ki(KS-1 downto KS-WS+3); 
     -- krr4 <= ki(KS-WS+3 downto KS-WS) & ki(KS-1 downto KS-WS+4); 
     
     ko   <= ( (not ki(WS-1 downto 0)) xor (ki(KS-WS+2 downto KS-WS) & ki(KS-1 downto KS-WS+3))  
         xor (ki(KS-WS+3 downto KS-WS) & ki(KS-1 downto KS-WS+4)) xor 
        (b"00000000000000000000000000000000000000000000000" & z(to_integer(62 - unsigned(round)))) 
        xor (x"000000000003")) & ki(KS-1 downto KS-WS) when (unsigned(round) > 0) 
        else ki;  
 
end behavior;